/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "arealensinversiongafactorybase.h"
#include "arealensinversiongafactoryparams.h"
#include "gridlensinversiongenomebase.h"

#include "debugnew.h"

namespace grale
{

AreaLensInversionGAFactoryBase::AreaLensInversionGAFactoryBase()
{
	currentparams = 0;
	maxgenerations = 0;
}

AreaLensInversionGAFactoryBase::~AreaLensInversionGAFactoryBase()
{
	if (currentparams)
		delete currentparams;
}

mogal::GAFactoryParams *AreaLensInversionGAFactoryBase::createParamsInstance() const
{
	return new AreaLensInversionGAFactoryParams();
}

const mogal::GAFactoryParams *AreaLensInversionGAFactoryBase::getCurrentParameters() const
{
	return currentparams;
}

bool AreaLensInversionGAFactoryBase::init(const mogal::GAFactoryParams *p)
{
	if (currentparams != 0)
	{
		setErrorString("Already initialized");
		return false;
	}

	if (p == 0)
	{
		setErrorString("Specified parameters can't be null");
		return false;
	}
	
	const LensInversionGAFactoryParams *p0 = (const LensInversionGAFactoryParams *)p;
	if (p0->getInversionType() != LensInversionGAFactoryParams::AreaInversion)
	{
		setErrorString("This algorithm is an area based one");
		return false;
	}

	const AreaLensInversionGAFactoryParams *p2 = (const AreaLensInversionGAFactoryParams *)p;
	
	currentparams = p2->createCopy();
	maxgenerations = currentparams->getMaximumNumberOfGenerations();

	if (!subInit(p2))
	{
		delete currentparams;
		currentparams = 0;
		return false;
	}
	return true;
}

} // end namespace
