/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#ifndef GRALE_POINTSORT_H

#define GRALE_POINTSORT_H

#include "graleconfig.h"
#include "vector2d.h"

namespace grale
{
	
template<class T>
class PointSort
{
public:
	PointSort()										{ m_x = 0; m_y = 0; }
	PointSort(Vector2D<T> p)								{ m_x = p.getX(); m_y = p.getY(); }
	void calculateScore(Vector2D<T> refpoint)						{ T a = m_x - refpoint.getX(); T b = m_y - refpoint.getY(); T s2 = a*a + b*b; m_score = a*ABS(a)/s2; }
	double getScore() const									{ return m_score; } 
	Vector2D<T> getVector2D() const								{ return Vector2D<T>(m_x, m_y); }
private:
	T m_x, m_y;
	T m_score;
};

template<class T>
inline bool operator< (const PointSort<T> &p1, const PointSort<T> &p2) // for the sorting algorithm
{
	return (p1.getScore() > p2.getScore());
}

} // end namespace

#endif // GRALE_POINTSORT_H
