/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#ifndef GRALE_MONOPOLEGRID2_H

#define GRALE_MONOPOLEGRID2_H

#include "graleconfig.h"
#include "monopolebasisfunction.h"
#include <errut/errorbase.h>
#include <serut/serializationinterface.h>
#include <vector>
#include <list>

namespace grale
{

class ImagesData;
class GravitationalLens;
	
class GRALE_IMPORTEXPORT MonopoleGrid2 : public errut::ErrorBase
{
public:
	MonopoleGrid2();
	~MonopoleGrid2();

	bool init(const ImagesData &avoidImages, const ImagesData &deflectionPoints, 
		  const Real2DFunction &target, const GravitationalLens &differenceDeflection,
	          Vector2D<double> center, double areaSize, int subDiv = 32, double zeroPointFactor = 2.0);

	int getNumberOfBasisFunctions() const						{ return m_monoPoleFunctions.size(); }
	void setBasisWeights(const std::vector<double> &w) 				{ if (w.size() == m_monoPoleWeights.size()) m_monoPoleWeights = w; }

	double getZeroPoint(int i) const						{ return m_monoPoleFunctions[i].getZeroPoint(); }
	double getLengthScale(int i) const						{ return m_monoPoleFunctions[i].getLengthScale(); }
	Vector2D<double> getPosition(int i) const					{ return m_monoPoleCenters[i]; }

	void prepareFitnessCalculation();
	void calculateFitness(double *pTotalDeflection, double *pNegative, double scale);

	bool write(serut::SerializationInterface &si) const;
	bool read(serut::SerializationInterface &si);
private:
	void clear();

	std::vector<MonopoleBasisFunction> m_monoPoleFunctions;
	std::vector<Vector2D<double> > m_monoPoleCenters;
	std::vector<double> m_monoPoleWeights;
	std::vector<std::vector<int> > m_pixelDependencies;
	std::vector<std::vector<int> > m_pointDependencies; // on which monopoles each deflection point depends
	std::vector<std::vector<Vector2D<double> > > m_pointMonoPoleDeflections; // deflections caused by each monopole at a specific point

	std::vector<double> m_targetMap;
	std::vector<double> m_monoPoleMap;

	std::vector<Vector2D<double> > m_differenceDeflections; // at the points
	std::vector<Vector2D<double> > m_monoPoleTotalDelections; // at the points

	Vector2D<double> m_bottomLeft;
	double m_stepSize;
	int m_subDiv;
};
	
} // end namespace

#endif // GRALE_MONOPOLEGRID2_H
